@@ -4,19 +4,19 @@ use strict;
 use warnings;
 
 use Carp;
-use Module::Load qw( load );
+use Mouse::Util::TypeConstraints ();
+
+our $VERSION = "0.01";
+
+*_get_isa_type_constraint  = \&Mouse::Util::TypeConstraints::find_or_create_isa_type_constraint;
+*_get_does_type_constraint = \&Mouse::Util::TypeConstraints::find_or_create_does_type_constraint;
 
-our $VERSION = "0.03_02";
 our $VERBOSE = 1;
 
-our $TYPE_CLASS = 'Class::Accessor::Typed::Mouse';
-
 my %key_ctor = (
-    rw      => \&_mk_accessors,
-    ro      => \&_mk_ro_accessors,
-    wo      => \&_mk_wo_accessors,
-    rw_lazy => \&_mk_lazy_accessors,
-    ro_lazy => \&_mk_ro_lazy_accessors,
+    rw => \&_mk_accessors,
+    ro => \&_mk_ro_accessors,
+    wo => \&_mk_wo_accessors,
 );
 
 sub import {
@@ -24,38 +24,28 @@ sub import {
     my $pkg = caller(0);
 
     my %rules;
-
-    if (exists $args{type}) {
-        $TYPE_CLASS = 'Class::Accessor::Typed::' . $args{type};
-    }
-    load($TYPE_CLASS);
-
     for my $key (sort keys %key_ctor) {
         if (defined $args{$key}) {
             croak("value of the '$key' parameter should be an hashref") unless ref($args{$key}) eq 'HASH';
 
-            for my $n (sort keys %{ $args{$key} }) {
+            for my $n (sort keys %{$args{$key}}) {
                 my $rule = ref($args{$key}->{$n}) eq 'HASH'
                     ? $args{$key}->{$n} : { isa => $args{$key}->{$n} };
 
                 $rule->{type} = do {
                     if (defined $rule->{isa}) {
-                        $TYPE_CLASS->type($rule->{isa});
+                        _get_isa_type_constraint($rule->{isa});
                     } elsif (defined $rule->{does}) {
-                        $TYPE_CLASS->type_role($rule->{isa});
+                        _get_does_type_constraint($rule->{does});
                     }
                 };
-                $rule->{lazy} = ($key eq 'rw_lazy' or $key eq 'ro_lazy') ? 1 : 0;
-
                 $args{$key}->{$n} = $rule;
-                $rules{$n} = $rule;
+                $rules{$n}        = $rule;
             }
-            $key_ctor{$key}->($pkg, %{ $args{$key} });
+            $key_ctor{$key}->($pkg, %{$args{$key}});
         }
     }
-    return 1 if exists $args{new} && !$args{new};
-
-    _mk_new($pkg, %rules);
+    _mk_new($pkg, %rules) if $args{new};
     return 1;
 }
 
@@ -63,7 +53,7 @@ sub _mk_new {
     my $pkg = shift;
     no strict 'refs';
 
-    *{ $pkg . '::new' } = __m_new($pkg, @_);
+    *{$pkg . '::new'} = __m_new($pkg, @_);
 }
 
 sub _mk_accessors {
@@ -71,9 +61,20 @@ sub _mk_accessors {
     no strict 'refs';
 
     while (@_) {
-        my $n    = shift;
+        my $n = shift;
         my $rule = shift;
-        *{ $pkg . '::' . $n } = __m($n, $rule->{type});
+        *{$pkg . '::' . $n} = __m($n, $rule->{type});
+    }
+}
+
+sub _mk_rw_accessors {
+    my $pkg = shift;
+    no strict 'refs';
+
+    while (@_) {
+        my $n = shift;
+        my $rule = shift;
+        *{$pkg . '::' . $n} = __m($pkg, $n, $rule->{type});
     }
 }
 
@@ -82,9 +83,9 @@ sub _mk_ro_accessors {
     no strict 'refs';
 
     while (@_) {
-        my $n    = shift;
+        my $n = shift;
         my $rule = shift;
-        *{ $pkg . '::' . $n } = __m_ro($pkg, $n);
+        *{$pkg . '::' . $n} = __m_ro($pkg, $n);
     }
 }
 
@@ -93,53 +94,26 @@ sub _mk_wo_accessors {
     no strict 'refs';
 
     while (@_) {
-        my $n    = shift;
+        my $n = shift;
         my $rule = shift;
-        *{ $pkg . '::' . $n } = __m_wo($pkg, $n, $rule->{type});
+        *{$pkg . '::' . $n} = __m_wo($pkg, $n, $rule->{type});
     }
 }
 
-sub _mk_lazy_accessors {
-    my $pkg = shift;
-    no strict 'refs';
-
-    while (@_) {
-        my $n       = shift;
-        my $rule    = shift;
-        my $builder = $rule->{builder} || "_build_$n";
-        *{ $pkg . '::' . $n } = __m_lazy($n, $rule->{type}, $builder);
-    }
-}
-
-sub _mk_ro_lazy_accessors {
-    my $pkg = shift;
-    no strict 'refs';
-
-    while (@_) {
-        my $n       = shift;
-        my $rule    = shift;
-        my $builder = $rule->{builder} || "_build_$n";
-        *{ $pkg . '::' . $n } = __m_ro_lazy($pkg, $n, $rule->{type}, $builder);
-    }
-}
 
 sub __m_new {
-    my $pkg   = shift;
+    my $pkg = shift;
     my %rules = @_;
     no strict 'refs';
     return sub {
         my $klass = shift;
-        my %args  = (@_ == 1 && ref($_[0]) eq 'HASH' ? %{ $_[0] } : @_);
+        my %args = (@_ == 1 && ref($_[0]) eq 'HASH' ? %{$_[0]} : @_);
         my %params;
 
         for my $n (sort keys %rules) {
-            if (!exists $args{$n}) {
-                next if $rules{$n}->{lazy};
-
+            if (! exists $args{$n}) {
                 if ($rules{$n}->{default}) {
                     $args{$n} = $rules{$n}->{default};
-                } elsif ($rules{$n}->{optional}) {
-                    next;
                 } else {
                     error("missing mandatory parameter named '\$$n'");
                 }
@@ -149,7 +123,7 @@ sub __m_new {
 
         if (keys %args > keys %rules) {
             my $message = 'unknown arguments: ' . join ', ', sort grep { not exists $rules{$_} } keys %args;
-            warnings::warn(void => $message);
+            warnings::warn( void => $message );
         }
         bless \%params, $klass;
     };
@@ -159,7 +133,7 @@ sub __m {
     my ($n, $type) = @_;
 
     sub {
-        return $_[0]->{$n}                            if @_ == 1;
+        return $_[0]->{$n} if @_ == 1;
         return $_[0]->{$n} = _check($n, $type, $_[1]) if @_ == 2;
     };
 }
@@ -184,40 +158,13 @@ sub __m_wo {
     };
 }
 
-sub __m_lazy {
-    my ($n, $type, $builder) = @_;
-
-    sub {
-        if (@_ == 1) {
-            return $_[0]->{$n} if exists $_[0]->{$n};
-            return $_[0]->{$n} = _check($n, $type, $_[0]->$builder);
-        } elsif (@_ == 2) {
-            return $_[0]->{$n} = _check($n, $type, $_[1]);
-        }
-    };
-}
-
-sub __m_ro_lazy {
-    my ($pkg, $n, $type, $builder) = @_;
-
-    sub {
-        if (@_ == 1) {
-            return $_[0]->{$n} if exists $_[0]->{$n};
-            return $_[0]->{$n} = _check($n, $type, $_[0]->$builder);
-        }
-        my $caller = caller(0);
-        error("'$caller' cannot alter the value of '$n' on objects of class '$pkg'");
-    };
-}
-
 sub _check {
-    my $n     = shift;
-    my $type  = shift;
+    my $n = shift;
+    my $type = shift;
     my $value = shift;
 
     return $value unless defined $type;
     return $value if $type->check($value);
-
     if ($type->has_coercion) {
         $value = $type->coerce($value);
         return $value if $type->check($value);
@@ -260,17 +207,9 @@ Class::Accessor::Typed - Class::Accessor::Lite with Type
         wo => {
             hoge => 'Int',
         },
-        rw_lazy => {
-            foo_lazy => 'Str',
-        }
-        ro_lazy => {
-            bar_lazy => { isa => 'Int', builder => 'bar_lazy_builder' },
-        }
+        new => 1,
     );
 
-    sub _build_foo_lazy  { 'string' }
-    sub bar_lazy_builder { 'string' }
-
 =head1 DESCRIPTION
 
 Class::Accessor::Typed is variant of C<Class::Accessor::Lite>. It supports argument validation like C<Smart::Args>.
@@ -291,8 +230,7 @@ Setting of property is defined by hash reference that specifies property name as
 
 =item new => $true_of_false
 
-If value evaluates to false, the default constructor is not created.
-The other cases, Class::Accessor::Typed provides the default constructor automatically.
+If value evaluates to true, the default constructor is created.
 
 =item rw => \%name_and_option_of_the_properties
 
@@ -306,20 +244,11 @@ create a read-only accessor.
 
 create a write-only accessor.
 
-=item rw_lazy => \%name_and_option_of_the_properties
-
-create a read / write lazy accessor.
-
-=item ro_lazy => \%name_and_option_of_the_properties
-
-create a read-only lazy accessor.
-
 =back
 
 =head2 PROPERTY RULE
 
-Property rule can receive string of type name (e.g. C<Int>) or hash reference (with C<isa>/C<does>, C<default>, C<optional> and C<builder>).
-C<default> can only use on C<rw>, C<ro> and C<wo>, and C<builder> can only use on C<rw_lazy> and C<ro_lazy>.
+Property rule can receive string of type name (e.g. C<Int>) or hash reference (with C<isa>/C<does> and C<default>).
 
 =head1 SEE ALSO
 
